@@ -27,8 +27,7 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-// ----
-// Author: Craig Silverstein
+// ---
 //
 // This implements a uniform interface for all 6 hash implementations:
 //    dense_hashtable, dense_hash_map, dense_hash_set
@@ -46,15 +45,15 @@
 #ifndef UTIL_GTL_HASH_TEST_INTERFACE_H_
 #define UTIL_GTL_HASH_TEST_INTERFACE_H_
 
-#include "config.h"
-#include HASH_MAP_H    // for hash<>
-#include <functional>  // for equal_to<>
-#include <google/sparsehash/sparsehashtable.h>
-#include <google/sparse_hash_map>
-#include <google/sparse_hash_set>
-#include <google/sparsehash/densehashtable.h>
-#include <google/dense_hash_map>
-#include <google/dense_hash_set>
+#include <sparsehash/internal/sparseconfig.h>
+#include <functional>          // for equal_to<>
+#include <sparsehash/internal/sparsehashtable.h>
+#include <sparsehash/sparse_hash_map>
+#include <sparsehash/sparse_hash_set>
+#include <sparsehash/internal/densehashtable.h>
+#include <sparsehash/dense_hash_map>
+#include <sparsehash/dense_hash_set>
+#include HASH_FUN_H    // for hash<>
 
 _START_GOOGLE_NAMESPACE_
 
@@ -271,17 +270,18 @@ class BaseHashtableInterface {
 
   size_type count(const key_type& key) const { return ht_.count(key); }
 
-  pair<iterator, iterator> equal_range(const key_type& key) {
-    pair<typename HT::iterator, typename HT::iterator> r
+  std::pair<iterator, iterator> equal_range(const key_type& key) {
+    std::pair<typename HT::iterator, typename HT::iterator> r
         = ht_.equal_range(key);
-    return pair<iterator, iterator>(iterator(r.first, this),
+    return std::pair<iterator, iterator>(iterator(r.first, this),
                                     iterator(r.second, this));
   }
-  pair<const_iterator, const_iterator> equal_range(const key_type& key) const {
-    pair<typename HT::const_iterator, typename HT::const_iterator> r
+  std::pair<const_iterator, const_iterator> equal_range(const key_type& key)
+      const {
+    std::pair<typename HT::const_iterator, typename HT::const_iterator> r
         = ht_.equal_range(key);
-    return pair<const_iterator, const_iterator>(const_iterator(r.first, this),
-                                                const_iterator(r.second, this));
+    return std::pair<const_iterator, const_iterator>(
+        const_iterator(r.first, this), const_iterator(r.second, this));
   }
 
   const_iterator random_element(class ACMRandom* r) const {
@@ -291,9 +291,9 @@ class BaseHashtableInterface {
     return iterator(ht_.random_element(r), this);
   }
 
-  pair<iterator, bool> insert(const value_type& obj) {
-    pair<typename HT::iterator, bool> r = ht_.insert(obj);
-    return pair<iterator, bool>(iterator(r.first, this), r.second);
+  std::pair<iterator, bool> insert(const value_type& obj) {
+    std::pair<typename HT::iterator, bool> r = ht_.insert(obj);
+    return std::pair<iterator, bool>(iterator(r.first, this), r.second);
   }
   template <class InputIterator>
   void insert(InputIterator f, InputIterator l) {
@@ -328,6 +328,15 @@ class BaseHashtableInterface {
     return ht_ != other.ht_;
   }
 
+  template <typename ValueSerializer, typename OUTPUT>
+  bool serialize(ValueSerializer serializer, OUTPUT *fp) {
+    return ht_.serialize(serializer, fp);
+  }
+  template <typename ValueSerializer, typename INPUT>
+  bool unserialize(ValueSerializer serializer, INPUT *fp) {
+    return ht_.unserialize(serializer, fp);
+  }
+
   template <typename OUTPUT>
   bool write_metadata(OUTPUT *fp) {
     return ht_.write_metadata(fp);
@@ -364,6 +373,7 @@ class BaseHashtableInterface {
   virtual bool supports_brackets() const = 0;     // has a 'real' operator[]
   virtual bool supports_readwrite() const = 0;
   virtual bool supports_num_table_copies() const = 0;
+  virtual bool supports_serialization() const = 0;
 
  protected:
   HT ht_;
@@ -378,9 +388,9 @@ class BaseHashtableInterface {
 // ---------------------------------------------------------------------
 
 template <class Key, class T,
-          class HashFcn = HASH_NAMESPACE::hash<Key>,
-          class EqualKey = STL_NAMESPACE::equal_to<Key>,
-          class Alloc = libc_allocator_with_realloc<pair<const Key, T> > >
+          class HashFcn = SPARSEHASH_HASH<Key>,
+          class EqualKey = std::equal_to<Key>,
+          class Alloc = libc_allocator_with_realloc<std::pair<const Key, T> > >
 class HashtableInterface_SparseHashMap
     : public BaseHashtableInterface< sparse_hash_map<Key, T, HashFcn,
                                                      EqualKey, Alloc> > {
@@ -421,12 +431,16 @@ class HashtableInterface_SparseHashMap
   bool supports_brackets() const { return true; }
   bool supports_readwrite() const { return true; }
   bool supports_num_table_copies() const { return false; }
+  bool supports_serialization() const { return true; }
 
   void set_empty_key(const typename p::key_type& k) { }
   void clear_empty_key() { }
   typename p::key_type empty_key() const { return typename p::key_type(); }
+
   int num_table_copies() const { return 0; }
 
+  typedef typename ht::NopointerSerializer NopointerSerializer;
+
  protected:
   template <class K2, class T2, class H2, class E2, class A2>
   friend void swap(HashtableInterface_SparseHashMap<K2,T2,H2,E2,A2>& a,
@@ -456,8 +470,8 @@ void swap(HashtableInterface_SparseHashMap<K,T,H,E,A>& a,
 // ---------------------------------------------------------------------
 
 template <class Value,
-          class HashFcn = HASH_NAMESPACE::hash<Value>,
-          class EqualKey = STL_NAMESPACE::equal_to<Value>,
+          class HashFcn = SPARSEHASH_HASH<Value>,
+          class EqualKey = std::equal_to<Value>,
           class Alloc = libc_allocator_with_realloc<Value> >
 class HashtableInterface_SparseHashSet
     : public BaseHashtableInterface< sparse_hash_set<Value, HashFcn,
@@ -524,12 +538,16 @@ class HashtableInterface_SparseHashSet
   bool supports_brackets() const { return false; }
   bool supports_readwrite() const { return true; }
   bool supports_num_table_copies() const { return false; }
+  bool supports_serialization() const { return true; }
 
   void set_empty_key(const typename p::key_type& k) { }
   void clear_empty_key() { }
   typename p::key_type empty_key() const { return typename p::key_type(); }
+
   int num_table_copies() const { return 0; }
 
+  typedef typename ht::NopointerSerializer NopointerSerializer;
+
  protected:
   template <class K2, class H2, class E2, class A2>
   friend void swap(HashtableInterface_SparseHashSet<K2,H2,E2,A2>& a,
@@ -636,6 +654,7 @@ class HashtableInterface_SparseHashtable
   bool supports_brackets() const { return false; }
   bool supports_readwrite() const { return true; }
   bool supports_num_table_copies() const { return true; }
+  bool supports_serialization() const { return true; }
 
   void set_empty_key(const typename p::key_type& k) { }
   void clear_empty_key() { }
@@ -647,6 +666,8 @@ class HashtableInterface_SparseHashtable
 
   // TODO(csilvers): also support/test destructive_begin()/destructive_end()?
 
+  typedef typename ht::NopointerSerializer NopointerSerializer;
+
  protected:
   template <class V2, class K2, class HF2, class EK2, class SK2, class Eq2,
             class A2>
@@ -684,9 +705,9 @@ void swap(HashtableInterface_SparseHashtable<V,K,HF,EK,SK,Eq,A>& a,
 // value saying what the empty key is.
 
 template <class Key, class T, const Key& EMPTY_KEY,
-          class HashFcn = HASH_NAMESPACE::hash<Key>,
-          class EqualKey = STL_NAMESPACE::equal_to<Key>,
-          class Alloc = libc_allocator_with_realloc<pair<const Key, T> > >
+          class HashFcn = SPARSEHASH_HASH<Key>,
+          class EqualKey = std::equal_to<Key>,
+          class Alloc = libc_allocator_with_realloc<std::pair<const Key, T> > >
 class HashtableInterface_DenseHashMap
     : public BaseHashtableInterface< dense_hash_map<Key, T, HashFcn,
                                                     EqualKey, Alloc> > {
@@ -733,9 +754,11 @@ class HashtableInterface_DenseHashMap
   bool supports_brackets() const { return true; }
   bool supports_readwrite() const { return false; }
   bool supports_num_table_copies() const { return false; }
+  bool supports_serialization() const { return true; }
 
-  template <typename OUTPUT> bool write_metadata(OUTPUT *fp) { return false; }
-  template <typename INPUT> bool read_metadata(INPUT *fp) { return false; }
+  typedef typename ht::NopointerSerializer NopointerSerializer;
+  template <typename OUTPUT> bool write_metadata(OUTPUT *) { return false; }
+  template <typename INPUT> bool read_metadata(INPUT *) { return false; }
   template <typename OUTPUT> bool write_nopointer_data(OUTPUT *) {
     return false;
   }
@@ -776,8 +799,8 @@ void swap(HashtableInterface_DenseHashMap<K,T,Empty,H,E,A>& a,
 // value saying what the empty key is.
 
 template <class Value, const Value& EMPTY_KEY,
-          class HashFcn = HASH_NAMESPACE::hash<Value>,
-          class EqualKey = STL_NAMESPACE::equal_to<Value>,
+          class HashFcn = SPARSEHASH_HASH<Value>,
+          class EqualKey = std::equal_to<Value>,
           class Alloc = libc_allocator_with_realloc<Value> >
 class HashtableInterface_DenseHashSet
     : public BaseHashtableInterface< dense_hash_set<Value, HashFcn,
@@ -835,9 +858,11 @@ class HashtableInterface_DenseHashSet
   bool supports_brackets() const { return false; }
   bool supports_readwrite() const { return false; }
   bool supports_num_table_copies() const { return false; }
+  bool supports_serialization() const { return true; }
 
-  template <typename OUTPUT> bool write_metadata(OUTPUT *fp) { return false; }
-  template <typename INPUT> bool read_metadata(INPUT *fp) { return false; }
+  typedef typename ht::NopointerSerializer NopointerSerializer;
+  template <typename OUTPUT> bool write_metadata(OUTPUT *) { return false; }
+  template <typename INPUT> bool read_metadata(INPUT *) { return false; }
   template <typename OUTPUT> bool write_nopointer_data(OUTPUT *) {
     return false;
   }
@@ -960,9 +985,11 @@ class HashtableInterface_DenseHashtable
   bool supports_brackets() const { return false; }
   bool supports_readwrite() const { return false; }
   bool supports_num_table_copies() const { return true; }
+  bool supports_serialization() const { return true; }
 
-  template <typename OUTPUT> bool write_metadata(OUTPUT *fp) { return false; }
-  template <typename INPUT> bool read_metadata(INPUT *fp) { return false; }
+  typedef typename ht::NopointerSerializer NopointerSerializer;
+  template <typename OUTPUT> bool write_metadata(OUTPUT *) { return false; }
+  template <typename INPUT> bool read_metadata(INPUT *) { return false; }
   template <typename OUTPUT> bool write_nopointer_data(OUTPUT *) {
     return false;
   }
