@@ -27,23 +27,63 @@
 // (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-// ----
-// Author: Craig Silverstein
+// ---
 //
-// This tests common/densehashtable.h
-// This tests common/dense_hash_set.h
-// This tests common/dense_hash_map.h
-// This tests common/sparsehashtable.h
-// This tests common/sparse_hash_set.h
-// This tests common/sparse_hash_map.h
+// This tests densehashtable
+// This tests dense_hash_set
+// This tests dense_hash_map
+// This tests sparsehashtable
+// This tests sparse_hash_set
+// This tests sparse_hash_map
 //
-// This test replacess hashtable_unittest.cc, which was becoming
-// unreadable.
+// This test replaces hashtable_unittest.cc, which was becoming
+// unreadable.  This file is opaque but hopefully not unreadable -- at
+// least, not the tests!
 //
 // Note that since all these classes are templatized, it's important
 // to call every public method on the class: not just to make sure
 // they work, but to make sure they even compile.
 
+#include <sparsehash/internal/sparseconfig.h>
+#include <config.h>
+#include <math.h>
+#include <stddef.h>   // for size_t
+#include <stdlib.h>
+#include <string.h>
+#ifdef HAVE_STDINT_H
+# include <stdint.h>
+#endif   // for uintptr_t
+#include <iostream>
+#include <set>
+#include <sstream>
+#include <typeinfo>   // for class typeinfo (returned by typeid)
+#include <vector>
+#include <sparsehash/type_traits.h>
+#include <sparsehash/sparsetable>
+#include "hash_test_interface.h"
+#include "testutil.h"
+namespace testing = GOOGLE_NAMESPACE::testing;
+
+using std::cout;
+using std::pair;
+using std::set;
+using std::string;
+using std::vector;
+using GOOGLE_NAMESPACE::dense_hash_map;
+using GOOGLE_NAMESPACE::dense_hash_set;
+using GOOGLE_NAMESPACE::sparse_hash_map;
+using GOOGLE_NAMESPACE::sparse_hash_set;
+using GOOGLE_NAMESPACE::sparsetable;
+using GOOGLE_NAMESPACE::HashtableInterface_SparseHashMap;
+using GOOGLE_NAMESPACE::HashtableInterface_SparseHashSet;
+using GOOGLE_NAMESPACE::HashtableInterface_SparseHashtable;
+using GOOGLE_NAMESPACE::HashtableInterface_DenseHashMap;
+using GOOGLE_NAMESPACE::HashtableInterface_DenseHashSet;
+using GOOGLE_NAMESPACE::HashtableInterface_DenseHashtable;
+namespace sparsehash_internal = GOOGLE_NAMESPACE::sparsehash_internal;
+
+typedef unsigned char uint8;
+
 #ifdef _MSC_VER
 // Below, we purposefully test having a very small allocator size.
 // This causes some "type conversion too small" errors when using this
@@ -52,37 +92,6 @@
 #pragma warning(disable:4244)
 #endif
 
-#include "config.h"
-#include <math.h>
-#include <stdlib.h>
-#include <string.h>
-#include <iostream>
-#include <set>
-#include <typeinfo>   // for class typeinfo (returned by typeid)
-#include <vector>
-#include <google/type_traits.h>
-#include <google/sparsetable>
-#include "hash_test_interface.h"
-#include "testutil.h"
-
-using GOOGLE_NAMESPACE::sparsetable;
-using GOOGLE_NAMESPACE::sparse_hash_map;
-using GOOGLE_NAMESPACE::sparse_hash_set;
-using GOOGLE_NAMESPACE::dense_hash_map;
-using GOOGLE_NAMESPACE::dense_hash_set;
-using GOOGLE_NAMESPACE::HashtableInterface_SparseHashMap;
-using GOOGLE_NAMESPACE::HashtableInterface_SparseHashSet;
-using GOOGLE_NAMESPACE::HashtableInterface_SparseHashtable;
-using GOOGLE_NAMESPACE::HashtableInterface_DenseHashMap;
-using GOOGLE_NAMESPACE::HashtableInterface_DenseHashSet;
-using GOOGLE_NAMESPACE::HashtableInterface_DenseHashtable;
-namespace testing = GOOGLE_NAMESPACE::testing;
-using STL_NAMESPACE::cout;
-using STL_NAMESPACE::pair;
-using STL_NAMESPACE::set;
-using STL_NAMESPACE::string;
-using STL_NAMESPACE::vector;
-
 namespace {
 
 #ifndef _MSC_VER   // windows defines its own version
@@ -92,13 +101,12 @@ static string TmpFile(const char* basename) {
 }
 # else
 static string TmpFile(const char* basename) {
-  return string("/tmp/") + basename;
+  string kTmpdir = "/tmp";
+  return kTmpdir + "/" + basename;
 }
 # endif
 #endif
 
-typedef unsigned char uint8;
-
 // Used as a value in some of the hashtable tests.  It's just some
 // arbitrary user-defined type with non-trivial memory management.
 struct ValueType {
@@ -168,6 +176,10 @@ struct Hasher {
       hash = 33 * hash + a[i];
     return hash;
   }
+  size_t operator()(const int* a) const {
+    num_hashes_++;
+    return static_cast<size_t>(reinterpret_cast<uintptr_t>(a));
+  }
   bool operator()(int a, int b) const {
     num_compares_++;
     return a == b;
@@ -338,8 +350,67 @@ template<> pair<const char* const,ValueType> UniqueObjectHelper(int index) {
       UniqueObjectHelper<char*>(index), UniqueObjectHelper<ValueType>(index+1));
 }
 
+class ValueSerializer {
+ public:
+  bool operator()(FILE* fp, const int& value) {
+    return fwrite(&value, sizeof(value), 1, fp) == 1;
+  }
+  bool operator()(FILE* fp, int* value) {
+    return fread(value, sizeof(*value), 1, fp) == 1;
+  }
+  bool operator()(FILE* fp, const string& value) {
+    const int size = value.size();
+    return (*this)(fp, size) && fwrite(value.c_str(), size, 1, fp) == 1;
+  }
+  bool operator()(FILE* fp, string* value) {
+    int size;
+    if (!(*this)(fp, &size)) return false;
+    char* buf = new char[size];
+    if (fread(buf, size, 1, fp) != 1) {
+      delete[] buf;
+      return false;
+    }
+    new(value) string(buf, size);
+    delete[] buf;
+    return true;
+  }
+  template <typename OUTPUT>
+  bool operator()(OUTPUT* fp, const ValueType& v) {
+    return (*this)(fp, string(v.s()));
+  }
+  template <typename INPUT>
+  bool operator()(INPUT* fp, ValueType* v) {
+    string data;
+    if (!(*this)(fp, &data)) return false;
+    new(v) ValueType(data.c_str());
+    return true;
+  }
+  template <typename OUTPUT>
+  bool operator()(OUTPUT* fp, const char* const& value) {
+    // Just store the index.
+    return (*this)(fp, atoi(value));
+  }
+  template <typename INPUT>
+  bool operator()(INPUT* fp, const char** value) {
+    // Look up via index.
+    int index;
+    if (!(*this)(fp, &index)) return false;
+    *value = UniqueObjectHelper<char*>(index);
+    return true;
+  }
+  template <typename OUTPUT, typename First, typename Second>
+  bool operator()(OUTPUT* fp, std::pair<const First, Second>* value) {
+    return (*this)(fp, const_cast<First*>(&value->first))
+        && (*this)(fp, &value->second);
+  }
+  template <typename INPUT, typename First, typename Second>
+  bool operator()(INPUT* fp, const std::pair<const First, Second>& value) {
+    return (*this)(fp, value.first) && (*this)(fp, value.second);
+  }
+};
+
 template <typename HashtableType>
-class HashtableTest {
+class HashtableTest : public ::testing::Test {
  public:
   HashtableTest() : ht_() { }
   // Give syntactically-prettier access to UniqueObjectHelper.
@@ -429,27 +500,58 @@ namespace {
 // We need to define the same class 4 times due to limitations in the
 // testing framework.  Basically, we associate each class below with
 // the set of types we want to run tests on it with.
-typedef testing::TypeList6<INT_HASHTABLES> IntHashtables;
 template <typename HashtableType> class HashtableIntTest
     : public HashtableTest<HashtableType> { };
-TYPED_TEST_CASE_6(HashtableIntTest, IntHashtables);
-
-typedef testing::TypeList6<STRING_HASHTABLES> StringHashtables;
 template <typename HashtableType> class HashtableStringTest
     : public HashtableTest<HashtableType> { };
-TYPED_TEST_CASE_6(HashtableStringTest, StringHashtables);
-
-typedef testing::TypeList6<CHARSTAR_HASHTABLES> CharStarHashtables;
 template <typename HashtableType> class HashtableCharStarTest
     : public HashtableTest<HashtableType> { };
-TYPED_TEST_CASE_6(HashtableCharStarTest, CharStarHashtables);
-
-typedef testing::TypeList18<INT_HASHTABLES, STRING_HASHTABLES,
-                            CHARSTAR_HASHTABLES> AllHashtables;
 template <typename HashtableType> class HashtableAllTest
     : public HashtableTest<HashtableType> { };
+
+typedef testing::TypeList6<INT_HASHTABLES> IntHashtables;
+typedef testing::TypeList6<STRING_HASHTABLES> StringHashtables;
+typedef testing::TypeList6<CHARSTAR_HASHTABLES> CharStarHashtables;
+typedef testing::TypeList18<INT_HASHTABLES, STRING_HASHTABLES,
+                            CHARSTAR_HASHTABLES> AllHashtables;
+
+TYPED_TEST_CASE_6(HashtableIntTest, IntHashtables);
+TYPED_TEST_CASE_6(HashtableStringTest, StringHashtables);
+TYPED_TEST_CASE_6(HashtableCharStarTest, CharStarHashtables);
 TYPED_TEST_CASE_18(HashtableAllTest, AllHashtables);
 
+// ------------------------------------------------------------------------
+// First, some testing of the underlying infrastructure.
+
+TEST(HashtableCommonTest, HashMunging) {
+  const Hasher hasher;
+
+  // We don't munge the hash value on non-pointer template types.
+  {
+    const sparsehash_internal::sh_hashtable_settings<int, Hasher, size_t, 1>
+        settings(hasher, 0.0, 0.0);
+    const int v = 1000;
+    EXPECT_EQ(hasher(v), settings.hash(v));
+  }
+
+  {
+    // We do munge the hash value on pointer template types.
+    const sparsehash_internal::sh_hashtable_settings<int*, Hasher, size_t, 1>
+        settings(hasher, 0.0, 0.0);
+    int* v = NULL;
+    v += 0x10000;    // get a non-trivial pointer value
+    EXPECT_NE(hasher(v), settings.hash(v));
+  }
+  {
+    const sparsehash_internal::sh_hashtable_settings<const int*, Hasher,
+                                                     size_t, 1>
+        settings(hasher, 0.0, 0.0);
+    const int* v = NULL;
+    v += 0x10000;    // get a non-trivial pointer value
+    EXPECT_NE(hasher(v), settings.hash(v));
+  }
+}
+
 // ------------------------------------------------------------------------
 // If the first arg to TYPED_TEST is HashtableIntTest, it will run
 // this test on all the hashtable types, with key=int and value=int.
@@ -793,7 +895,7 @@ TYPED_TEST(HashtableAllTest, Swap) {
 
   // We purposefully don't swap allocs -- they're not necessarily swappable.
 
-  // Now swap back, using the free-function swap.
+  // Now swap back, using the free-function swap
   // NOTE: MSVC seems to have trouble with this free swap, not quite
   // sure why.  I've given up trying to fix it though.
 #ifdef _MSC_VER
@@ -810,6 +912,20 @@ TYPED_TEST(HashtableAllTest, Swap) {
   EXPECT_EQ(1u, other_ht.size());
   EXPECT_EQ(1u, this->ht_.count(this->UniqueKey(111)));
   EXPECT_EQ(0u, other_ht.count(this->UniqueKey(111)));
+
+  // A user reported a crash with this code using swap to clear.
+  // We've since fixed the bug; this prevents a regression.
+  TypeParam swap_to_clear_ht;
+  swap_to_clear_ht.set_deleted_key(this->UniqueKey(1));
+  for (int i = 2; i < 10000; ++i) {
+    swap_to_clear_ht.insert(this->UniqueObject(i));
+  }
+  TypeParam empty_ht;
+  empty_ht.swap(swap_to_clear_ht);
+  swap_to_clear_ht.set_deleted_key(this->UniqueKey(1));
+  for (int i = 2; i < 10000; ++i) {
+    swap_to_clear_ht.insert(this->UniqueObject(i));
+  }
 }
 
 TYPED_TEST(HashtableAllTest, Size) {
@@ -1085,6 +1201,7 @@ TYPED_TEST(HashtableAllTest, BracketInsert) {
   EXPECT_EQ(2, ht.hash_funct().num_hashes());
 }
 
+
 TYPED_TEST(HashtableAllTest, InsertValue) {
   // First, try some straightforward insertions.
   EXPECT_TRUE(this->ht_.empty());
@@ -1310,10 +1427,9 @@ TYPED_TEST(HashtableAllTest, Equals) {
 }
 
 TEST(HashtableTest, IntIO) {
-  // Since dense_hash_* doesn't support IO yet, and the set case is
-  // just a special (easier) case than the map case, I just test on
-  // sparse_hash_map.  This handles the easy case where we can use the
-  // standard reader and writer.
+  // Since the set case is just a special (easier) case than the map case, I
+  // just test on sparse_hash_map.  This handles the easy case where we can
+  // use the standard reader and writer.
   sparse_hash_map<int, int> ht_out;
   ht_out.set_deleted_key(0);
   for (int i = 1; i < 1000; i++) {
@@ -1345,10 +1461,9 @@ TEST(HashtableTest, IntIO) {
 }
 
 TEST(HashtableTest, StringIO) {
-  // Since dense_hash_* doesn't support IO yet, and the set case is
-  // just a special (easier) case than the map case, I just test on
-  // sparse_hash_map.  This handles the difficult case where we have
-  // to write our own custom reader/writer for the data.
+  // Since the set case is just a special (easier) case than the map case,
+  // I just test on sparse_hash_map.  This handles the difficult case where
+  // we have to write our own custom reader/writer for the data.
   sparse_hash_map<string, string, Hasher, Hasher> ht_out;
   ht_out.set_deleted_key(string(""));
   for (int i = 32; i < 128; i++) {
@@ -1405,6 +1520,195 @@ TEST(HashtableTest, StringIO) {
   EXPECT_EQ(string(""), ht_in["y"]);
 }
 
+TYPED_TEST(HashtableAllTest, Serialization) {
+  if (!this->ht_.supports_serialization()) return;
+  TypeParam ht_out;
+  ht_out.set_deleted_key(this->UniqueKey(2000));
+  for (int i = 1; i < 100; i++) {
+    ht_out.insert(this->UniqueObject(i));
+  }
+  // just to test having some erased keys when we write.
+  ht_out.erase(this->UniqueKey(56));
+  ht_out.erase(this->UniqueKey(22));
+
+  string file(TmpFile("serialization"));
+  FILE* fp = fopen(file.c_str(), "wb");
+  EXPECT_TRUE(fp != NULL);
+  EXPECT_TRUE(ht_out.serialize(ValueSerializer(), fp));
+  fclose(fp);
+
+  TypeParam ht_in;
+  fp = fopen(file.c_str(), "rb");
+  EXPECT_TRUE(fp != NULL);
+  EXPECT_TRUE(ht_in.unserialize(ValueSerializer(), fp));
+  fclose(fp);
+
+  EXPECT_EQ(this->UniqueObject(1), *ht_in.find(this->UniqueKey(1)));
+  EXPECT_EQ(this->UniqueObject(99), *ht_in.find(this->UniqueKey(99)));
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(100)));
+  EXPECT_EQ(this->UniqueObject(21), *ht_in.find(this->UniqueKey(21)));
+  // should not have been saved
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(22)));
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(56)));
+}
+
+TYPED_TEST(HashtableIntTest, NopointerSerialization) {
+  if (!this->ht_.supports_serialization()) return;
+  TypeParam ht_out;
+  ht_out.set_deleted_key(this->UniqueKey(2000));
+  for (int i = 1; i < 100; i++) {
+    ht_out.insert(this->UniqueObject(i));
+  }
+  // just to test having some erased keys when we write.
+  ht_out.erase(this->UniqueKey(56));
+  ht_out.erase(this->UniqueKey(22));
+
+  string file(TmpFile("nopointer_serialization"));
+  FILE* fp = fopen(file.c_str(), "wb");
+  EXPECT_TRUE(fp != NULL);
+  EXPECT_TRUE(ht_out.serialize(typename TypeParam::NopointerSerializer(), fp));
+  fclose(fp);
+
+  TypeParam ht_in;
+  fp = fopen(file.c_str(), "rb");
+  EXPECT_TRUE(fp != NULL);
+  EXPECT_TRUE(ht_in.unserialize(typename TypeParam::NopointerSerializer(), fp));
+  fclose(fp);
+
+  EXPECT_EQ(this->UniqueObject(1), *ht_in.find(this->UniqueKey(1)));
+  EXPECT_EQ(this->UniqueObject(99), *ht_in.find(this->UniqueKey(99)));
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(100)));
+  EXPECT_EQ(this->UniqueObject(21), *ht_in.find(this->UniqueKey(21)));
+  // should not have been saved
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(22)));
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(56)));
+}
+
+// We don't support serializing to a string by default, but you can do
+// it by writing your own custom input/output class.
+class StringIO {
+ public:
+  explicit StringIO(string* s) : s_(s) {}
+  size_t Write(const void* buf, size_t len) {
+    s_->append(reinterpret_cast<const char*>(buf), len);
+    return len;
+  }
+  size_t Read(void* buf, size_t len) {
+    if (s_->length() < len)
+      len = s_->length();
+    memcpy(reinterpret_cast<char*>(buf), s_->data(), len);
+    s_->erase(0, len);
+    return len;
+  }
+ private:
+  string* const s_;
+};
+
+TYPED_TEST(HashtableIntTest, SerializingToString) {
+  if (!this->ht_.supports_serialization()) return;
+  TypeParam ht_out;
+  ht_out.set_deleted_key(this->UniqueKey(2000));
+  for (int i = 1; i < 100; i++) {
+    ht_out.insert(this->UniqueObject(i));
+  }
+  // just to test having some erased keys when we write.
+  ht_out.erase(this->UniqueKey(56));
+  ht_out.erase(this->UniqueKey(22));
+
+  string stringbuf;
+  StringIO stringio(&stringbuf);
+  EXPECT_TRUE(ht_out.serialize(typename TypeParam::NopointerSerializer(),
+                               &stringio));
+
+  TypeParam ht_in;
+  EXPECT_TRUE(ht_in.unserialize(typename TypeParam::NopointerSerializer(),
+                                &stringio));
+
+  EXPECT_EQ(this->UniqueObject(1), *ht_in.find(this->UniqueKey(1)));
+  EXPECT_EQ(this->UniqueObject(99), *ht_in.find(this->UniqueKey(99)));
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(100)));
+  EXPECT_EQ(this->UniqueObject(21), *ht_in.find(this->UniqueKey(21)));
+  // should not have been saved
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(22)));
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(56)));
+}
+
+// An easier way to do the above would be to use the existing stream methods.
+TYPED_TEST(HashtableIntTest, SerializingToStringStream) {
+  if (!this->ht_.supports_serialization()) return;
+  TypeParam ht_out;
+  ht_out.set_deleted_key(this->UniqueKey(2000));
+  for (int i = 1; i < 100; i++) {
+    ht_out.insert(this->UniqueObject(i));
+  }
+  // just to test having some erased keys when we write.
+  ht_out.erase(this->UniqueKey(56));
+  ht_out.erase(this->UniqueKey(22));
+
+  std::stringstream string_buffer;
+  EXPECT_TRUE(ht_out.serialize(typename TypeParam::NopointerSerializer(),
+                               &string_buffer));
+
+  TypeParam ht_in;
+  EXPECT_TRUE(ht_in.unserialize(typename TypeParam::NopointerSerializer(),
+                                &string_buffer));
+
+  EXPECT_EQ(this->UniqueObject(1), *ht_in.find(this->UniqueKey(1)));
+  EXPECT_EQ(this->UniqueObject(99), *ht_in.find(this->UniqueKey(99)));
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(100)));
+  EXPECT_EQ(this->UniqueObject(21), *ht_in.find(this->UniqueKey(21)));
+  // should not have been saved
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(22)));
+  EXPECT_FALSE(ht_in.count(this->UniqueKey(56)));
+}
+
+// Verify that the metadata serialization is endianness and word size
+// agnostic.
+TYPED_TEST(HashtableAllTest, MetadataSerializationAndEndianness) {
+  TypeParam ht_out;
+  string kExpectedDense("\x13W\x86""B\0\0\0\0\0\0\0 \0\0\0\0\0\0\0\0\0\0\0\0",
+                        24);
+  string kExpectedSparse("$hu1\0\0\0 \0\0\0\0\0\0\0\0\0\0\0\0", 20);
+
+  if (ht_out.supports_readwrite()) {
+    string file(TmpFile("metadata_serialization"));
+    FILE* fp = fopen(file.c_str(), "wb");
+    EXPECT_TRUE(fp != NULL);
+
+    EXPECT_TRUE(ht_out.write_metadata(fp));
+    EXPECT_TRUE(ht_out.write_nopointer_data(fp));
+
+    const size_t num_bytes = ftell(fp);
+    fclose(fp);
+    fp = fopen(file.c_str(), "rb");
+    EXPECT_LE(num_bytes, static_cast<size_t>(24));
+    char contents[24];
+    EXPECT_EQ(num_bytes, fread(contents, 1, num_bytes, fp));
+    EXPECT_EQ(EOF, fgetc(fp));       // check we're *exactly* the right size
+    fclose(fp);
+    // TODO(csilvers): check type of ht_out instead of looking at the 1st byte.
+    if (contents[0] == kExpectedDense[0]) {
+      EXPECT_EQ(kExpectedDense, string(contents, num_bytes));
+    } else {
+      EXPECT_EQ(kExpectedSparse, string(contents, num_bytes));
+    }
+  }
+
+  // Do it again with new-style serialization.  Here we can use StringIO.
+  if (ht_out.supports_serialization()) {
+    string stringbuf;
+    StringIO stringio(&stringbuf);
+    EXPECT_TRUE(ht_out.serialize(typename TypeParam::NopointerSerializer(),
+                                 &stringio));
+    if (stringbuf[0] == kExpectedDense[0]) {
+      EXPECT_EQ(kExpectedDense, stringbuf);
+    } else {
+      EXPECT_EQ(kExpectedSparse, stringbuf);
+    }
+  }
+}
+
+
 // ------------------------------------------------------------------------
 // The above tests test the general API for correctness.  These tests
 // test a few corner cases that have tripped us up in the past, and
@@ -1606,20 +1910,12 @@ TEST(HashtableTest, NestedHashtables) {
 
 TEST(HashtableDeathTest, ResizeOverflow) {
   dense_hash_map<int, int> ht;
-  try {
-    ht.resize(static_cast<size_t>(-1));
-    EXPECT_TRUE(false && "dense_hash_map reszie should have failed");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the resize failed.
-  }
+  EXPECT_DEATH(ht.resize(static_cast<size_t>(-1)),
+               "overflows size_type");
 
   sparse_hash_map<int, int> ht2;
-  try {
-    ht2.resize(static_cast<size_t>(-1));
-    EXPECT_TRUE(false && "sparse_hash_map reszie should have failed");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the resize failed.
-  }
+  EXPECT_DEATH(ht2.resize(static_cast<size_t>(-1)),
+               "overflows size_type");
 }
 
 TEST(HashtableDeathTest, InsertSizeTypeOverflow) {
@@ -1638,18 +1934,10 @@ TEST(HashtableDeathTest, InsertSizeTypeOverflow) {
   EXPECT_TRUE(dhs.get_allocator().is_custom_alloc());
 
   // Test size_type overflow in insert(it, it)
-  try {
-    dhs.insert(test_data.begin(), test_data.end());
-    EXPECT_TRUE(false && "dense_hash_map::insert(it,it) should have overflown");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the operation failed.
-  }
-  try {
-    shs.insert(test_data.begin(), test_data.end());
-    EXPECT_TRUE(false && "sparse_hash_map::insert(it,it) should have overflown");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the operation failed.
-  }
+  EXPECT_DEATH(dhs.insert(test_data.begin(), test_data.end()),
+               "overflows size_type");
+  EXPECT_DEATH(shs.insert(test_data.begin(), test_data.end()),
+               "overflows size_type");
 }
 
 TEST(HashtableDeathTest, InsertMaxSizeOverflow) {
@@ -1664,18 +1952,10 @@ TEST(HashtableDeathTest, InsertMaxSizeOverflow) {
   dhs.set_empty_key(-1);
 
   // Test max_size overflow
-  try {
-    dhs.insert(test_data.begin(), test_data.begin() + 11);
-    EXPECT_TRUE(false && "dense_hash_map max_size check should have failed");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the operation failed.
-  }
-  try {
-    shs.insert(test_data.begin(), test_data.begin() + 11);
-    EXPECT_TRUE(false && "sparse_hash_map max_size check should have failed");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the operation failed.
-  }
+  EXPECT_DEATH(dhs.insert(test_data.begin(), test_data.begin() + 11),
+               "exceed max_size");
+  EXPECT_DEATH(shs.insert(test_data.begin(), test_data.begin() + 11),
+               "exceed max_size");
 }
 
 TEST(HashtableDeathTest, ResizeSizeTypeOverflow) {
@@ -1684,18 +1964,8 @@ TEST(HashtableDeathTest, ResizeSizeTypeOverflow) {
   dense_hash_set<int, Hasher, Hasher, Alloc<int, uint8, 10> > dhs;
   dhs.set_empty_key(-1);
 
-  try {
-    dhs.resize(250);  // 9+250 > 25
-    EXPECT_TRUE(false && "dense_hash_map resize should have seen overflow");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the operation failed.
-  }
-  try {
-    shs.resize(250);
-    EXPECT_TRUE(false && "sparse_hash_map resize should have seen overflow");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the operation failed.
-  }
+  EXPECT_DEATH(dhs.resize(250), "overflows size_type");  // 9+250 > 256
+  EXPECT_DEATH(shs.resize(250), "overflows size_type");
 }
 
 TEST(HashtableDeathTest, ResizeDeltaOverflow) {
@@ -1712,18 +1982,10 @@ TEST(HashtableDeathTest, ResizeDeltaOverflow) {
     dhs.insert(i);
     shs.insert(i);
   }
-  try {
-    dhs.insert(test_data.begin(), test_data.begin() + 250);
-    EXPECT_TRUE(false && "dense_hash_map big insert should have overflowed");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the operation failed.
-  }
-  try {
-    shs.insert(test_data.begin(), test_data.begin() + 250);
-    EXPECT_TRUE(false && "sparse_hash_map big insert should have overflowed");
-  } catch (const STL_NAMESPACE::length_error&) {
-    // Good, the operation failed.
-  }
+  EXPECT_DEATH(dhs.insert(test_data.begin(), test_data.begin() + 250),
+               "overflows size_type");              // 9+250 > 256
+  EXPECT_DEATH(shs.insert(test_data.begin(), test_data.begin() + 250),
+               "overflows size_type");
 }
 
 // ------------------------------------------------------------------------
@@ -1731,8 +1993,8 @@ TEST(HashtableDeathTest, ResizeDeltaOverflow) {
 // Also, benchmarks.
 
 TYPED_TEST(HashtableAllTest, ClassSizes) {
-  cout << "sizeof(" << typeid(TypeParam).name() << "): "
-       << sizeof(this->ht_) << "\n";
+  std::cout << "sizeof(" << typeid(TypeParam).name() << "): "
+            << sizeof(this->ht_) << "\n";
 }
 
 }  // unnamed namespace
