@@ -1,10 +1,10 @@
 // Copyright (c) 2005, Google Inc.
 // All rights reserved.
-// 
+//
 // Redistribution and use in source and binary forms, with or without
 // modification, are permitted provided that the following conditions are
 // met:
-// 
+//
 //     * Redistributions of source code must retain the above copyright
 // notice, this list of conditions and the following disclaimer.
 //     * Redistributions in binary form must reproduce the above
@@ -14,7 +14,7 @@
 //     * Neither the name of Google Inc. nor the names of its
 // contributors may be used to endorse or promote products derived from
 // this software without specific prior written permission.
-// 
+//
 // THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 // "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 // LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
@@ -28,32 +28,31 @@
 // OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 // ---
-// Author: Craig Silverstein
-//
-// This tests <google/sparsetable>
 //
 // Since sparsetable is templatized, it's important that we test every
 // function in every class in this file -- not just to see if it
 // works, but even if it compiles.
 
-#include "config.h"
-#include <string>
+#include <sparsehash/internal/sparseconfig.h>
+#include <config.h>
 #include <stdio.h>
-#include <string.h>         // for memcmp()
+#include <string.h>
+#include <sys/types.h>      // for size_t
 #include <stdlib.h>         // defines unlink() on some windows platforms(?)
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>         // for unlink()
-#endif
-#include <sys/types.h>      // for size_t
-#include <string>
+# include <unistd.h>
+#endif         // for unlink()
 #include <memory>           // for allocator
-#include <google/sparsetable>
-
-using STL_NAMESPACE::string;
-using STL_NAMESPACE::allocator;
+#include <string>
+#include <sparsehash/sparsetable>
+using std::string;
+using std::allocator;
 using GOOGLE_NAMESPACE::sparsetable;
 using GOOGLE_NAMESPACE::DEFAULT_SPARSEGROUP_SIZE;
 
+typedef u_int16_t uint16;
+string FLAGS_test_tmpdir = "/tmp/";
+
 // Many sparsetable operations return a size_t.  Rather than have to
 // use PRIuS everywhere, we'll just cast to a "big enough" value.
 #define UL(x)    ( static_cast<unsigned long>(x) )
@@ -273,6 +272,28 @@ void TestInt() {
   TEST(x.begin() == x.begin() + 1 - 1);
   TEST(z.begin() != z.end());
 
+  // ----------------------------------------------------------------------
+  // Test the non-empty iterators get_pos function
+
+  sparsetable<unsigned int> gp(100);
+  for (int i = 0; i < 100; i += 9) {
+    gp.set(i,i);
+  }
+
+  for (sparsetable<unsigned int>::const_nonempty_iterator
+           it = gp.nonempty_begin(); it != gp.nonempty_end(); ++it) {
+    out += snprintf(out, LEFT,
+                    "get_pos() for const nonempty_iterator: %u == %lu\n",
+                    *it, UL(gp.get_pos(it)));
+  }
+
+  for (sparsetable<unsigned int>::nonempty_iterator
+           it = gp.nonempty_begin(); it != gp.nonempty_end(); ++it) {
+    out += snprintf(out, LEFT,
+                    "get_pos() for nonempty_iterator: %u == %lu\n",
+                    *it, UL(gp.get_pos(it)));
+  }
+
   // ----------------------------------------------------------------------
   // Test sparsetable functions
   out += snprintf(out, LEFT, "x has %lu/%lu buckets, "
@@ -280,7 +301,7 @@ void TestInt() {
                   UL(x.num_nonempty()), UL(x.size()),
                   UL(y.num_nonempty()), UL(y.size()),
                   UL(z.num_nonempty()), UL(z.size()));
-  
+
   y.resize(48);              // should get rid of 48 and 49
   y.resize(70);              // 48 and 49 should still be gone
   out += snprintf(out, LEFT, "y shrank and grew: it's now %lu/%lu\n",
@@ -322,8 +343,8 @@ void TestInt() {
     out += snprintf(out, LEFT, "y[??] = %d\n", *--it);
 
   // ----------------------------------------------------------------------
-  // Test I/O
-  string filestr = "/tmp/.sparsetable.test";
+  // Test I/O using deprecated read/write_metadata
+  string filestr = FLAGS_test_tmpdir + "/.sparsetable.test";
   const char *file = filestr.c_str();
   FILE *fp = fopen(file, "wb");
   if ( fp == NULL ) {
@@ -354,6 +375,31 @@ void TestInt() {
     out += snprintf(out, LEFT, "That's %lu set buckets\n", UL(y2.num_nonempty()));
   }
   unlink(file);
+
+  // ----------------------------------------------------------------------
+  // Also test I/O using serialize()/unserialize()
+  fp = fopen(file, "wb");
+  if ( fp == NULL ) {
+    out += snprintf(out, LEFT, "Can't open %s, skipping disk write...\n", file);
+  } else {
+    y.serialize(sparsetable<int>::NopointerSerializer(), fp);
+    fclose(fp);
+  }
+  fp = fopen(file, "rb");
+  if ( fp == NULL ) {
+    out += snprintf(out, LEFT, "Can't open %s, skipping disk read...\n", file);
+  } else {
+    sparsetable<int> y2;
+    y2.unserialize(sparsetable<int>::NopointerSerializer(), fp);
+    fclose(fp);
+
+    for ( sparsetable<int>::const_iterator it = y2.begin(); it != y2.end(); ++it ) {
+      if ( y2.test(it) )
+        out += snprintf(out, LEFT, "y2[%lu] is %d\n", UL(it - y2.begin()), *it);
+    }
+    out += snprintf(out, LEFT, "That's %lu set buckets\n", UL(y2.num_nonempty()));
+  }
+  unlink(file);
 }
 
 // Test sparsetable with a non-POD type, std::string
@@ -462,7 +508,7 @@ void ResetAllocatorCounters() {
 template <class T> class instrumented_allocator {
  public:
   typedef T value_type;
-  typedef u_int16_t size_type;
+  typedef uint16 size_type;
   typedef ptrdiff_t difference_type;
 
   typedef T* pointer;
@@ -758,6 +804,30 @@ static const char g_expected[] = (
     "first non-empty x: 10\n"
     "x.begin() == x.begin() + 1 - 1? yes\n"
     "z.begin() != z.end()? no\n"
+    "get_pos() for const nonempty_iterator: 0 == 0\n"
+    "get_pos() for const nonempty_iterator: 9 == 9\n"
+    "get_pos() for const nonempty_iterator: 18 == 18\n"
+    "get_pos() for const nonempty_iterator: 27 == 27\n"
+    "get_pos() for const nonempty_iterator: 36 == 36\n"
+    "get_pos() for const nonempty_iterator: 45 == 45\n"
+    "get_pos() for const nonempty_iterator: 54 == 54\n"
+    "get_pos() for const nonempty_iterator: 63 == 63\n"
+    "get_pos() for const nonempty_iterator: 72 == 72\n"
+    "get_pos() for const nonempty_iterator: 81 == 81\n"
+    "get_pos() for const nonempty_iterator: 90 == 90\n"
+    "get_pos() for const nonempty_iterator: 99 == 99\n"
+    "get_pos() for nonempty_iterator: 0 == 0\n"
+    "get_pos() for nonempty_iterator: 9 == 9\n"
+    "get_pos() for nonempty_iterator: 18 == 18\n"
+    "get_pos() for nonempty_iterator: 27 == 27\n"
+    "get_pos() for nonempty_iterator: 36 == 36\n"
+    "get_pos() for nonempty_iterator: 45 == 45\n"
+    "get_pos() for nonempty_iterator: 54 == 54\n"
+    "get_pos() for nonempty_iterator: 63 == 63\n"
+    "get_pos() for nonempty_iterator: 72 == 72\n"
+    "get_pos() for nonempty_iterator: 81 == 81\n"
+    "get_pos() for nonempty_iterator: 90 == 90\n"
+    "get_pos() for nonempty_iterator: 99 == 99\n"
     "x has 3/7 buckets, y 4/70, z 0/0\n"
     "y shrank and grew: it's now 2/70\n"
     "y[12] = -12, y.get(12) = -12\n"
@@ -803,6 +873,19 @@ static const char g_expected[] = (
     "y2[37] is -37\n"
     "y2[9898] is -9898\n"
     "That's 12 set buckets\n"
+    "y2[10] is -10\n"
+    "y2[11] is -11\n"
+    "y2[13] is -13\n"
+    "y2[14] is -14\n"
+    "y2[30] is -30\n"
+    "y2[31] is -31\n"
+    "y2[32] is -32\n"
+    "y2[33] is -33\n"
+    "y2[35] is -35\n"
+    "y2[36] is -36\n"
+    "y2[37] is -37\n"
+    "y2[9898] is -9898\n"
+    "That's 12 set buckets\n"
     "string test\n"
     "x[0]: \n"
     "x[1]: \n"
@@ -873,7 +956,10 @@ static const char g_expected[] = (
     );
 
 // defined at bottom of file for ease of maintainence
-int main(int /*argc*/, char ** /*argv*/) {
+int main(int argc, char **argv) {          // though we ignore the args
+  (void)argc;
+  (void)argv;
+
   TestInt();
   TestString();
   TestAllocator();
